using ElectricLocomotive.DrawningObject;
using ElectricLocomotive.MovementStrategy;
using ProjectElectricLocomotive;
using ProjectElectricLocomotive.MovementStrategy;
using System;

namespace ElectricLocomotive
{
    public partial class ElectricLocomotive : Form
    {

        private DrawningLocomotive? _drawningLocomotive;
        private AbstractStrategy? _abstractStrategy;
        public DrawningLocomotive? SelectedLocomotive { get; private set; }

        public ElectricLocomotive()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedLocomotive = null;
        }

        private void Draw()
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxElectroLocomotiv.Width, pictureBoxElectroLocomotiv.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningLocomotive.DrawTransport(gr);
            pictureBoxElectroLocomotiv.Image = bmp;
        }

        private void PaintLocoButton_click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
                color = dialog.Color;
            _drawningLocomotive = new DrawningLocomotive(random.Next(100, 300), random.Next(1000, 3000), color, pictureBoxElectroLocomotiv.Width, pictureBoxElectroLocomotiv.Height);
            _drawningLocomotive.SetPosition(random.Next(10, 100), random.Next(10, 100));

            Draw();
        }

        private void PaintElectricLocoButton_click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            bool Horns_1 = Convert.ToBoolean(random.Next(0, 2));
            bool Horns_2 = Convert.ToBoolean(random.Next(0, 2));

            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
                color = dialog.Color;
            if (dialog.ShowDialog() == DialogResult.OK)
                dopColor = dialog.Color;
            _drawningLocomotive = new DrawningElectricLocomotive(random.Next(100, 300), random.Next(1000, 3000), color, dopColor, Horns_1,Horns_2, pictureBoxElectroLocomotiv.Width, pictureBoxElectroLocomotiv.Height);
            _drawningLocomotive.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningLocomotive.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningLocomotive.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningLocomotive.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningLocomotive.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToRightCorner(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
                DrawningObjectLocomotive(_drawningLocomotive), pictureBoxElectroLocomotiv.Width,
                pictureBoxElectroLocomotiv.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void ButtonSelectLocomotive_Click(object sender, EventArgs e)
        {
            SelectedLocomotive = _drawningLocomotive;
            DialogResult = DialogResult.OK;
        }
    }
}