﻿using ElectricLocomotive;
using ElectricLocomotive.DrawningObject;
using ElectricLocomotive.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectElectricLocomotive.DrawingObjects
{
    public static class ExtentionLocomotive
    {
        /// <summary>
        /// Создание объекта из строки
        /// </summary>
        /// <param name="info">Строка с данными для создания объекта</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <param name="width">Ширина</param>
        /// <param name="height">Высота</param>
        /// <returns>Объект</returns>
        public static DrawningLocomotive? CreateDrawningLocomotive(this string info, char
        separatorForObject, int width, int height)
        {
            string[] strs = info.Split(separatorForObject);
            if (strs.Length == 3)
            {
                return new DrawningLocomotive(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]),
                Color.FromName(strs[2]), width, height);
            }
            if (strs.Length == 6)
            {
                return new DrawningElectricLocomotive(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]),
                Color.FromName(strs[2]),
                Color.FromName(strs[3]),
                Convert.ToBoolean(strs[4]),
                Convert.ToBoolean(strs[5]), width, height);
            }
            return null;
        }
        /// <summary>
        /// Получение данных для сохранения в файл
        /// </summary>
        /// <param name="drawningCar">Сохраняемый объект</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <returns>Строка с данными по объекту</returns>
        public static string GetDataForSave(this DrawningLocomotive drawningCar, char separatorForObject)
        {
            var loco = drawningCar.EntityLocomotive;
            if (loco == null)
            {
                return string.Empty;
            }
            var str =
            $"{loco.Speed}{separatorForObject}{loco.Weight}{separatorForObject}{loco.BodyColor.Name}";
            if (loco is not EntityElectroLocomotive electricLocomotive)
            {
                return str;
            }
            return 
            $"{str}{separatorForObject}{electricLocomotive.AdditionalColor.Name}{separatorForObject}{electricLocomotive.Horns_1}{separatorForObject}{electricLocomotive.Horns_2}";
        }
    }
}
