﻿using ElectricLocomotive.DrawningObject;
using Microsoft.VisualBasic.Devices;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ElectricLocomotive
{
    public partial class FormLocomotiveConfig : Form
    {
        DrawningLocomotive? _locomotive = null;
        private event Action<DrawningLocomotive>? EventAddLocomotive;

        public FormLocomotiveConfig()
        {
            InitializeComponent();
            panelColorBlack.MouseDown += PanelColor_MouseDown;
            panelColorPurple.MouseDown += PanelColor_MouseDown;
            panelColorGray.MouseDown += PanelColor_MouseDown;
            panelColorGreen.MouseDown += PanelColor_MouseDown;
            panelColorRed.MouseDown += PanelColor_MouseDown;
            panelColorWhite.MouseDown += PanelColor_MouseDown;
            panelColorYellow.MouseDown += PanelColor_MouseDown;
            panelColorBlue.MouseDown += PanelColor_MouseDown;

            buttonCancelObject.Click += (s, e) => Close();

        }
        public void AddEvent(Action<DrawningLocomotive> ev)
        {
            if (EventAddLocomotive == null)
            {
                EventAddLocomotive = ev;
            }
            else
            {
                EventAddLocomotive += ev;
            }
        }
        private void DrawLocomotive()
        {
            Bitmap bmp = new(pictureBoxLoco.Width, pictureBoxLoco.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _locomotive?.SetPosition(5, 5);
            _locomotive?.DrawTransport(gr);
            pictureBoxLoco.Image = bmp;
        }
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _locomotive = new DrawningLocomotive((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.Blue, pictureBoxLoco.Width,
                   pictureBoxLoco.Height);
                    break;
                case "labelAdvancedObject":
                    _locomotive = new DrawningElectricLocomotive((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.Blue, Color.Red, checkBoxPantograph_1.Checked, checkBoxPantograph_2.Checked,
                   pictureBoxLoco.Width, pictureBoxLoco.Height);
                    break;
            }
            DrawLocomotive();
        }

        private void buttonAddObject_Click(object sender, EventArgs e)
        {
            EventAddLocomotive?.Invoke(_locomotive);
            Close();
        }

        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_locomotive == null)
                return;
            switch (((Label)sender).Name)
            {
                case "labelSimpleColor":
                    _locomotive.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelAdvancedColor":
                    if (!(_locomotive is DrawningElectricLocomotive))
                    {
                        return;
                    }
                    (_locomotive as DrawningElectricLocomotive).SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawLocomotive();
        }
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if ((e.Data?.GetDataPresent(typeof(Color)) ?? false))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
}
